#!/bin/sh

BARNYARDCONF=/etc/barnyard2/barnyard2.conf
MYSQL_PASSWD="$(sed -n 's;^output[[:blank:]]\+database:.*mysql.*password=\([^[:blank:]]\+\).*;\1;p' "$BARNYARDCONF")"
MYSQL_COMM="/usr/bin/mysql --skip-column-names --batch -u snort --password='$MYSQL_PASSWD' snort"
ALTERATOR_SNORT_CONF="/etc/sysconfig/alterator-snort"

[ -r "$ALTERATOR_SNORT_CONF" ] && . $ALTERATOR_SNORT_CONF

num_to_ipv4addr()
{
	num="$1"; shift

	printf '%s.%s.%s.%s\n' \
		"$(($num >> 24 & 0xff))" \
		"$(($num >> 16 & 0xff))" \
		"$(($num >> 8 & 0xff))" \
		"$(($num & 0xff))"
}

check_db()
{
	local IFS=$'\n'

	for sensor_sid in $(echo "SELECT sid FROM sensor;" | eval "$MYSQL_COMM"); do
		sid="$(echo "SELECT sid FROM alterator_last WHERE sid='$sensor_sid';" | eval "$MYSQL_COMM")"
		[ -z "$sid" ] || continue

		echo "INSERT INTO alterator_last(sid, last_cid) VALUES ('$sensor_sid', '0');" | eval "$MYSQL_COMM"
	done
}

check_event()
{
	local IFS=$'\n'
	local email_id="$1"
	local sid="$2"
	local last_cid="$3"
	local max_cid="$4"

	[ -n "$email_id" ] || return
	[ -n "$sid" ] || return
	[ -n "$last_cid" ] || return
	[ -n "$max_cid" ] || return

	for line in $(echo "SELECT s.sig_name, sg.sig_class_name, COUNT(e.cid), i.ip_src, i.ip_dst FROM event e
				LEFT JOIN signature s ON e.signature=s.sig_id
				LEFT JOIN sig_class sg ON s.sig_class_id=sg.sig_class_id
				LEFT JOIN iphdr i ON e.sid=i.sid AND e.cid=i.cid
					WHERE	e.sid='$sid' AND
						e.cid>'$last_cid' AND
						e.cid<='$max_cid' AND
						s.sig_class_id IN (SELECT sig_class_id FROM alterator_notifications WHERE email_id='$email_id')
							GROUP BY e.signature, i.ip_src, i.ip_dst;" | eval "$MYSQL_COMM"); do
		sig_name="$(echo "$line" | cut -f1)"
		sig_class_name="$(echo "$line" | cut -f2)"
		count="$(echo "$line" | cut -f3)"
		ip_from="$(num_to_ipv4addr "$(echo "$line" | cut -f4)")"
		ip_to="$(num_to_ipv4addr "$(echo "$line" | cut -f5)")"

		echo "$sig_class_name: $sig_name [$ip_from -> $ip_to] ($count)"
	done
}

main()
{
	check_db

	local IFS=$'\n'

	for i in $(echo "SELECT	sid,
				last_cid,
				(SELECT cid FROM event WHERE sid=alterator_last.sid ORDER BY cid DESC LIMIT 1),
				(SELECT hostname FROM sensor WHERE sid=alterator_last.sid),
				(SELECT interface FROM sensor WHERE sid=alterator_last.sid) FROM alterator_last;" | eval "$MYSQL_COMM"); do
		sid="$(echo "$i" | cut -f1)"
		last_cid="$(echo "$i" | cut -f2)"
		max_cid="$(echo "$i" | cut -f3)"
		sensor_hostname="$(echo "$i" | cut -f4)"
		sensor_interface="$(echo "$i" | cut -f5)"

		for j in $(echo "SELECT id, email FROM alterator_email;" | eval "$MYSQL_COMM"); do
			email_id="$(echo "$j" | cut -f1)"
			email="$(echo "$j" | cut -f2)"

			text_body=$(check_event "$email_id" "$sid" "$last_cid" "$max_cid")
			[ -n "$text_body" ] || continue;

			addparam=""
			[ -z "$NOTIFICATIONS_SENDER" ] || addparam="-f $NOTIFICATIONS_SENDER"
			echo "Subject: [alterator-snort] $sensor_hostname:$sensor_interface"$'\n'$'\n'"$text_body" | /usr/sbin/sendmail $addparam "$email"
		done

		echo "UPDATE alterator_last SET last_cid='$max_cid' WHERE sid='$sid';" | eval "$MYSQL_COMM"
	done
}

main

