(define-module (ui snort ban ajax)
    :use-module (alterator str)
    :use-module (alterator ajax)
    :use-module (alterator woo)
    :export (on-load))

(define (set-status)
  (for-each (lambda(p) (form-update-value p ""))
            '("label_listen_at"
              "label_stopped_at"
              "label_b_listen_at"
              "label_b_stopped_at"))
  (catch/message
    (lambda()
      (let* ((data (woo-read-first "/snort/status" 'language (form-value "language"))))
        (form-update-value-list '("label_listen_at"
                                  "label_stopped_at"
                                  "label_b_listen_at"
                                  "label_b_stopped_at")
                                data)))))

(define (update-rule-list)
  (form-update-enum "rule_list" (woo-call "/snort/ban_list"))
  (form-update-value "upd_enabled" "#f")
  (form-update-value "upd_bantime" "")
  (form-update-value "upd_findtime" "")
  (form-update-value "upd_maxretry" "")
  (form-update-enum "upd_in_classes" '())
  (form-update-enum "upd_out_classes" '()))

(define (ui-add-rule)
  (let ((name (form-value "add_name")))
    (if (not-empty-string? name)
      (catch/message
        (lambda()
          (woo-call "/snort/ban_add" 'name name)
          (form-update-value "add_name" "")
          (update-rule-list))))))

(define (read-rule)
  (let ((rule (form-value "rule_list"))
        (params '("upd_enabled" "upd_bantime" "upd_findtime" "upd_maxretry")))
    (if rule
      (begin
        (form-update-value-list params (woo-call "/snort/ban_read" 'rule rule))
        (form-update-enum "upd_in_classes" (woo-call "/snort/ban_read_in_classes" 'rule rule))
        (form-update-enum "upd_out_classes" (woo-call "/snort/ban_read_out_classes" 'rule rule))))))

(define (ui-upd-add-class)
  (let ((new_classes (form-value "upd_out_classes"))
        (rule (form-value "rule_list")))
    (if (and new_classes rule)
      (catch/message
        (lambda()
          (woo-call "/snort/ban_write_in_classes" 'rule rule 'new_classes new_classes)
          (form-update-enum "upd_in_classes" (woo-call "/snort/ban_read_in_classes" 'rule rule))
          (form-update-enum "upd_out_classes" (woo-call "/snort/ban_read_out_classes" 'rule rule)))))))

(define (ui-upd-remove-class)
  (let ((del_classes (form-value "upd_in_classes"))
        (rule (form-value "rule_list")))
    (if (and del_classes rule)
      (catch/message
        (lambda()
          (woo-call "/snort/ban_write_out_classes" 'rule rule 'del_classes del_classes)
          (form-update-enum "upd_in_classes" (woo-call "/snort/ban_read_in_classes" 'rule rule))
          (form-update-enum "upd_out_classes" (woo-call "/snort/ban_read_out_classes" 'rule rule)))))))

(define (ui-upd-apply)
  (let ((rule (form-value "rule_list")))
    (if rule
      (catch/message
        (lambda()
          (woo-call "/snort/ban_write" 'rule rule
                                       'upd_enabled (form-value "upd_enabled")
                                       'upd_bantime (form-value "upd_bantime")
                                       'upd_findtime (form-value "upd_findtime")
                                       'upd_maxretry (form-value "upd_maxretry"))
          (read-rule))))))

(define (ui-upd-delete)
  (let ((rule (form-value "rule_list")))
    (if rule
      (catch/message
        (lambda()
          (woo-call "/snort/ban_remove" 'rule rule)
          (update-rule-list))))))

(define (ui-reload-fail2ban)
  (catch/message
    (lambda()
      (woo-call "/snort/reload_fail2ban"))))

(define (on-load)
  (update-rule-list)
  (set-status)

  (form-bind "rule_list" "change" read-rule)

  (form-bind "add_rule_btn" "click" ui-add-rule)
  (form-bind "upd_add_class" "click" ui-upd-add-class)
  (form-bind "upd_remove_class" "click" ui-upd-remove-class)
  (form-bind "upd_apply_btn" "click" ui-upd-apply)
  (form-bind "upd_delete_btn" "click" ui-upd-delete)
  (form-bind "reload_fail2ban_btn" "click" ui-reload-fail2ban))
