(define-module (ui snort base ajax)
    :use-module (alterator str)
    :use-module (alterator ajax)
    :use-module (alterator woo)
    :use-module (alterator effect)
    :export (on-load))

(define (ui-generate)
  (form-update-value-list
    '("passwd_auto")
    (car (woo-call "/snort/pass_generate"))))

(define (ui-reset-password)
  (form-update-value "passwd_1" "")
  (form-update-value "passwd_2" "")
  (form-update-value "auto" #f)
  (ui-generate)
  (update-effect))

(define (ui-user-add)
  (let ((new_login (form-value "new_login"))
        (new_name (form-value "new_name"))
        (new_password (form-value "new_password")))
    (catch/message
      (lambda()
        (woo-call "/snort/base_user_add" 'new_login new_login 'new_name new_name 'new_password new_password)
        (form-update-value "new_login" "")
        (form-update-value "new_name" "")
        (form-update-value "new_password" "")
        (ui-update-user-list)))))

(define (ui-update-user-list)
  (let* ((data (woo-call "/snort/base_user_list")))
    (form-update-enum "user_list" data)))

(define (ui-read-user)
  (let* ((user (form-value "user_list"))
         (params '("upd_name")))
    (if user
      (begin
        (form-update-value-list
          params
          (woo-call "/snort/base_user_data" 'user user))
        (ui-reset-password)))))

(define (ui-user-apply)
  (let ((user (form-value "user_list"))
        (upd_name (form-value "upd_name")))
    (catch/message
      (lambda()
        (woo-call "/snort/base_user_apply" 'user user
                                           'upd_name upd_name
                                           'auto (form-value "auto")
                                           'passwd_1 (form-value "passwd_1")
                                           'passwd_2 (form-value "passwd_2")
                                           'passwd_auto (form-value "passwd_auto"))
        (ui-read-user)))))

(define (ui-user-del)
  (let ((user (form-value "user_list")))
    (catch/message
      (lambda()
        (woo-call "/snort/base_user_del" 'user user)
        (form-update-value "upd_name" "")
        (ui-update-user-list)))))

(define (set-status)
  (for-each (lambda(p) (form-update-value p ""))
            '("label_listen_at"
              "label_stopped_at"
              "label_b_listen_at"
              "label_b_stopped_at"))
  (catch/message
    (lambda()
      (let* ((data (woo-read-first "/snort/status" 'language (form-value "language"))))
        (form-update-value-list '("label_listen_at"
                                  "label_stopped_at"
                                  "label_b_listen_at"
                                  "label_b_stopped_at")
                                data)))))

(define (on-load)
  (effect-show "passwd_1" "auto" #f)
  (effect-show "passwd_2" "auto" #f)
  (effect-show "passwd_auto" "auto" #t)
  (effect-show "generate_button" "auto" #t)
  (init-effect)

  (ui-update-user-list)
  (set-status)

  (form-bind "user_list" "change" ui-read-user)

  (form-bind "useradd" "click" ui-user-add)
  (form-bind "userapply" "click" ui-user-apply)
  (form-bind "userdel" "click" ui-user-del)
  (form-bind "generate_button" "click" ui-generate))
