(define-module (ui snort config ajax)
    :use-module (alterator ajax)
    :use-module (alterator woo)
    :use-module (alterator effect)
    :export (init))

(define (read-state)
  (catch/message
    (lambda()
      (form-update-value-list '("state_enabled") (woo-read-first "/snort/state_enabled"
                                                              'language (form-value "language"))))))

(define (read-iface-list)
  (if (string=? (form-value "interfaces") "specific")
      (begin (form-update-enum "in_ifaces" (woo-call "/snort/config_in_ifaces"))
             (form-update-enum "out_ifaces" (woo-call "/snort/config_out_ifaces")))))

(define (set-status)
  (for-each (lambda(p) (form-update-value p ""))
            '("label_listen_at"
              "label_stopped_at"
              "label_b_listen_at"
              "label_b_stopped_at"))
  (catch/message
    (lambda()
      (let* ((data (woo-read-first "/snort/status" 'language (form-value "language"))))
        (form-update-value-list '("label_listen_at"
                                  "label_stopped_at"
                                  "label_b_listen_at"
                                  "label_b_stopped_at")
                                data)))))

(define (read-iface-mode)
  (set-status)
  (form-update-value-list '("interfaces") (woo-call "/snort/config_ifaces_mode"))
  (read-iface-list)
  (update-effect))

(define (ui-init)
  (read-state)
  (read-iface-mode))

(define (on-apply-state)
  ; always do reset, in success - read real status
  (for-each (lambda(p) (form-update-value p ""))
            '("label_listen_at"
              "label_stopped_at"
              "label_b_listen_at"
              "label_b_stopped_at"))

  (let ((state (form-value "state_enabled")))
    (form-update-value "state_enabled" #f)
    (catch/message
      (lambda()
        (woo-write "/snort/state_enabled"
                   'state_enabled state
                   'language (form-value "language")))))

    (set-status)
    (read-state))

(define (ui-change-ifaces-mode)
  (let ((mode (form-value "interfaces")))
       (if (string=? mode "any")
           (begin (woo-call "/snort/config_ifaces_change" 'ifaces "any")
                  (form-update-enum "in_ifaces" '())
                  (form-update-enum "out_ifaces" '()))
           (read-iface-list))))

(define (ui-add-ifaces)
  (let ((newifaces (form-value "out_ifaces")))
    (if (string? newifaces)
      (catch/message
	(lambda()
	  (woo-call "/snort/config_ifaces_add" 'newifaces newifaces))))
    (read-iface-list)))

(define (ui-remove-ifaces)
  (let ((delifaces (form-value "in_ifaces")))
    (if (string? delifaces)
      (catch/message
	(lambda()
	  (woo-call "/snort/config_ifaces_del" 'delifaces delifaces))))
    (read-iface-list)))

(define (init)
  (effect-enable "in_ifaces" "interfaces" "specific")
  (effect-enable "out_ifaces" "interfaces" "specific")
  (effect-enable "add_ifaces" "interfaces" "specific")
  (effect-enable "remove_ifaces" "interfaces" "specific")
  (init-effect)

  (form-bind "apply_state_button" "click" on-apply-state)
  (form-bind "add_ifaces" "click" ui-add-ifaces)
  (form-bind "remove_ifaces" "click" ui-remove-ifaces)

  (form-bind "interfaces" "change" ui-change-ifaces-mode)
  (ui-init))
