(define-module (ui snort notifications ajax)
    :use-module (alterator str)
    :use-module (alterator ajax)
    :use-module (alterator woo)
    :use-module (alterator effect)
    :export (on-load))

(define (set-status)
  (for-each (lambda(p) (form-update-value p ""))
            '("label_listen_at"
              "label_stopped_at"
              "label_b_listen_at"
              "label_b_stopped_at"))
  (catch/message
    (lambda()
      (let* ((data (woo-read-first "/snort/status" 'language (form-value "language"))))
        (form-update-value-list '("label_listen_at"
                                  "label_stopped_at"
                                  "label_b_listen_at"
                                  "label_b_stopped_at")
                                data)))))

(define (ui-read)
  (let ((email (form-value "email_list")))
    (if email
      (begin
        (update-type-in-list email)
        (update-type-out-list email)))))

(define (ui-list)
  (let* ((data (woo-call "/snort/notifications_list_email")))
    (form-update-enum "email_list" data)
    (ui-read)))

(define (ui-delete)
  (let ((email (form-value "email_list")))
    (if (and email (form-confirm (string-append (_ "Do you really want to delete email?"))))
      (catch/message
	(lambda()
	  (woo-call "/snort/notifications_destroy_email" 'email email)
	  (ui-list)
	  (form-update-enum "in_types" '())
	  (form-update-enum "out_types" '()))))))

(define (ui-new)
  (let ((add_email (form-value "add_email")))
    (if (not-empty-string? add_email)
      (catch/message
	(lambda()
	  (woo-call "/snort/notifications_create_email" 'add_email add_email)
	  (form-update-value "add_email" "")
	  (ui-list))))))

(define (update-type-in-list . email)
  (and email (form-update-enum "in_types" (woo-list "/snort/notifications_in_types" 'email email))))

(define (update-type-out-list . email)
  (and email (form-update-enum "out_types" (woo-list "/snort/notifications_out_types" 'email email))))

(define (ui-add-to-type)
  (let ((newtype (form-value "out_types"))
	(email (form-value "email_list")))
    (if (and (string? newtype) email)
      (catch/message
	(lambda()
	  (woo-call "/snort/notifications_add_type" 'email email 'newtype newtype)
	  (update-type-in-list email)
	  (update-type-out-list email))))))

(define (ui-del-from-type)
  (let ((deltype (form-value "in_types"))
	(email (form-value "email_list")))
    (if (and (string? deltype) email)
      (catch/message
	(lambda()
	  (woo-call "/snort/notifications_del_type" 'email email 'deltype deltype)
	  (update-type-in-list email)
	  (update-type-out-list email))))))

(define (read-notifications-data)
  (form-update-value-list '("period" "weekday" "monthday" "time")
                          (woo-call "/snort/notifications_read_data" 'language (form-value "language")))
  (update-effect))

(define (on-notifications-apply)
  (catch/message
    (lambda()
      (apply woo-call "/snort/notifications_write_data"
             (form-value-list
               '("period" "weekday" "monthday" "time" "language")))))
  (read-notifications-data))

(define (ui-read-sender)
  (let ((params '("sender")))
    (form-update-value-list
          params
          (woo-call "/snort/notifications_read_sender"))))

(define (ui-apply-sender)
  (let ((sender (form-value "sender")))
    (catch/message
      (lambda()
	(woo-call "/snort/notifications_apply_sender" 'sender sender)
	(ui-read-sender)))))

(define (ui-send-test-mail)
  (let ((email (form-value "email_list")))
    (if email
      (catch/message
	(lambda()
	  (woo-call "/snort/notifications_send_test_mail" 'email email))))))

(define (on-load)
  (effect-enable "time" "period" "daily")
  (effect-enable "time" "period" "weekly")
  (effect-enable "time" "period" "monthly")
  (effect-enable "weekday" "period" "weekly")
  (effect-enable "monthday" "period" "monthly")

  (init-effect)

  (ui-list)

  (form-bind "email_list" "change" ui-read)

  (form-bind "delete_button" "click" ui-delete)
  (form-bind "add_button" "click" ui-new)
  (form-bind "add_to_type" "click" ui-add-to-type)
  (form-bind "remove_from_type" "click" ui-del-from-type)
  (form-bind "notifications_apply_button" "click" on-notifications-apply)
  (form-bind "sender_button" "click" ui-apply-sender)
  (form-bind "send_test_mail_button" "click" ui-send-test-mail)

  (form-update-enum "in_types" '())
  (form-update-enum "out_types" '())

  (form-update-enum "weekday" (woo-list "/snort/avail_weekday" 'language (form-value "language")))

  (read-notifications-data)

  (ui-read-sender)
  (set-status))
