(define-module (ui snort rules ajax)
    :use-module (alterator ajax)
    :use-module (alterator woo)
    :use-module (alterator effect)
    :use-module (srfi srfi-1)
    :use-module (srfi srfi-2)
    :use-module (srfi srfi-11)
    :export (init))

; DEBUG STRING (PASTE IN FOR YOUR NEEDS).
; (format #t "XXX: ~S\n" rdp_profile_name)
; (format #t "[ajax, module] action info\n")

; TO DEBUG DO:
; 1) define *debug* to #t
; 2) start ahttpd -l -d
(define *debug* #f)
(define (dmsg msg . args)
  (if *debug*
    (format #t "[ajax, net-l2tp]: ~S ~S\n" msg args)))

(define (update-rules-list rlist)
  (form-update-enum "in_rules" rlist))

(define (update-rules-unused-list rlist)
  (form-update-enum "out_rules" rlist))

(define (read-rules-list)
  (catch/message
    (lambda()
      (update-rules-list (woo-list "/snort/avail_rules" 'language (form-value "language"))))))

(define (read-rules-unused-list)
  (catch/message
    (lambda()
      (update-rules-unused-list (woo-list "/snort/avail_unused_rules" 'language (form-value "language"))))))

(define (update-activity)
  (let ((val (form-value "auto_update")))
    (form-update-activity "weekday" (and val
                                         (string=? "weekly" (form-value "period"))))
    (form-update-activity "monthday" (and val
                                          (string=? "monthly" (form-value "period"))))))

(define (ui-read)
  (read-rules-list)
  (read-rules-unused-list))

(define (set-status)
  (for-each (lambda(p) (form-update-value p ""))
            '("label_listen_at"
              "label_stopped_at"
              "label_b_listen_at"
              "label_b_stopped_at"))
  (catch/message
    (lambda()
      (let* ((data (woo-read-first "/snort/status" 'language (form-value "language"))))
        (form-update-value-list '("label_listen_at"
                                  "label_stopped_at"
                                  "label_b_listen_at"
                                  "label_b_stopped_at")
                                data)))))

(define (read-download-data)
  (set-status)
  (form-update-value-list '("rules_url" "oinkcode" "custom_url" "auto_update" "period" "weekday" "monthday" "time")
                          (woo-read-first "/snort/download-data" 'language (form-value "language")))
  (update-effect)
  (update-activity))

(define (ui-init)
  (ui-read)
  (form-update-enum "weekday" (woo-list "/snort/avail_weekday" 'language (form-value "language")))
  (read-download-data))

(define (on-download-apply)
  (catch/message
    (lambda()
      (apply woo-write "/snort/download-data"
             (form-value-list
               '("rules_url" "oinkcode" "custom_url" "auto_update" "period" "weekday" "monthday" "time" "language")))))
  (read-download-data))

(define (on-download)
  (catch/message
    (lambda()
      (woo-write "/snort/download-now"
                 'language (form-value "language"))))
  (ui-read))

(define (ui-add-rules)
  (let ((newrules (form-value "out_rules")))
    (if (string? newrules)
      (catch/message
	(lambda()
	  (woo-call "/snort/rules_add" 'newrules newrules))))
    (ui-read)))

(define (ui-remove-rules)
  (let ((delrules (form-value "in_rules")))
    (if (string? delrules)
      (catch/message
	(lambda()
	  (woo-call "/snort/rules_remove" 'delrules delrules))))
    (ui-read)))

(define (ui-reload-snort)
  ; always do reset, in success - read real status
  (for-each (lambda(p) (form-update-value p ""))
            '("label_listen_at"
              "label_stopped_at"
              "label_b_listen_at"
              "label_b_stopped_at"))
  (catch/message
    (lambda()
      (woo-call "/snort/reload_snort")))
  (set-status))

(define (init)
  (effect-enable "period" "auto_update" #t)
  (effect-enable "time" "auto_update" #t)
  (effect-enable "weekday" "period" "weekly")
  (effect-enable "monthday" "period" "monthly")
  (effect-enable "oinkcode" "rules_url" "oinkcode")
  (effect-enable "custom_url" "rules_url" "custom")
  (effect-disable "auto_update" "rules_url" #f) 
  (init-effect)
  (form-bind "auto_update" "change" update-activity)
  (form-bind "download_apply_button" "click" on-download-apply)
  (form-bind "download_button" "click" on-download)
  (form-bind "add_rules" "click" ui-add-rules)
  (form-bind "remove_rules" "click" ui-remove-rules)
  (form-bind "reload_snort_btn" "click" ui-reload-snort)
  (ui-init))
